function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

import React, { useContext, useEffect, useRef, useState } from 'react';
import isEqual from 'react-fast-compare';
import objToKey from './objToKey';
import { getCachedObservableQuery, invalidateCachedObservableQuery } from './queryCache';
var ApolloContext = React.createContext();
export function ApolloProvider(_ref) {
  var children = _ref.children,
      client = _ref.client;
  return React.createElement(ApolloContext.Provider, {
    value: client
  }, children);
}
export function useApolloClient() {
  return useContext(ApolloContext);
}
export function useQuery(query, _temp) {
  var _ref2 = _temp === void 0 ? {} : _temp,
      _ref2$suspend = _ref2.suspend,
      suspend = _ref2$suspend === void 0 ? true : _ref2$suspend,
      restOptions = _objectWithoutPropertiesLoose(_ref2, ["suspend"]);

  var client = useApolloClient();

  var _useState = useState(),
      result = _useState[0],
      setResult = _useState[1];

  var previousQuery = useRef();
  var previousRestOptions = useRef();
  var observableQuery = useRef();
  useEffect(function () {
    var subscription = observableQuery.current.subscribe(function (nextResult) {
      setResult(nextResult);
    });
    invalidateCachedObservableQuery(client, query, restOptions);
    return function () {
      subscription.unsubscribe();
    };
  }, [query, objToKey(restOptions)]);
  ensureSupportedFetchPolicy(restOptions.fetchPolicy, suspend);
  var helpers = {
    fetchMore: function fetchMore(opts) {
      return observableQuery.current.fetchMore(opts);
    },
    refetch: function refetch(variables) {
      return observableQuery.current.refetch(variables);
    },
    startPolling: function startPolling(interval) {
      return observableQuery.current.startPolling(interval);
    },
    stopPolling: function stopPolling() {
      return observableQuery.current.stopPolling();
    },
    updateQuery: function updateQuery(updaterFn) {
      return observableQuery.current.updateQuery(updaterFn);
    }
  };

  if (!(query === previousQuery.current && isEqual(restOptions, previousRestOptions.current))) {
    previousQuery.current = query;
    previousRestOptions.current = restOptions;
    var watchedQuery = getCachedObservableQuery(client, query, restOptions);
    observableQuery.current = watchedQuery;
    var currentResult = watchedQuery.currentResult();

    if (currentResult.partial && suspend) {
      // throw a promise - use the react suspense to wait until the data is
      // available
      throw watchedQuery.result();
    }

    setResult(currentResult);
    return _extends({}, helpers, currentResult);
  }

  return _extends({}, helpers, result);
}
export function useMutation(mutation, baseOptions) {
  var client = useApolloClient();
  return function (localOptions) {
    return client.mutate(_extends({
      mutation: mutation
    }, baseOptions, localOptions));
  };
}

function ensureSupportedFetchPolicy(fetchPolicy, suspend) {
  if (!suspend) {
    return;
  }

  if (fetchPolicy && fetchPolicy !== 'cache-first') {
    throw new Error("Fetch policy " + fetchPolicy + " is not supported without 'suspend: false'");
  }
}