"use strict";

exports.__esModule = true;
exports.ApolloProvider = ApolloProvider;
exports.useApolloClient = useApolloClient;
exports.useQuery = useQuery;
exports.useMutation = useMutation;

var _react = _interopRequireWildcard(require("react"));

var _reactFastCompare = _interopRequireDefault(require("react-fast-compare"));

var _objToKey = _interopRequireDefault(require("./objToKey"));

var _queryCache = require("./queryCache");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

var ApolloContext = _react.default.createContext();

function ApolloProvider(_ref) {
  var children = _ref.children,
      client = _ref.client;
  return _react.default.createElement(ApolloContext.Provider, {
    value: client
  }, children);
}

function useApolloClient() {
  return (0, _react.useContext)(ApolloContext);
}

function useQuery(query, _temp) {
  var _ref2 = _temp === void 0 ? {} : _temp,
      _ref2$suspend = _ref2.suspend,
      suspend = _ref2$suspend === void 0 ? true : _ref2$suspend,
      restOptions = _objectWithoutPropertiesLoose(_ref2, ["suspend"]);

  var client = useApolloClient();

  var _useState = (0, _react.useState)(),
      result = _useState[0],
      setResult = _useState[1];

  var previousQuery = (0, _react.useRef)();
  var previousRestOptions = (0, _react.useRef)();
  var observableQuery = (0, _react.useRef)();
  (0, _react.useEffect)(function () {
    var subscription = observableQuery.current.subscribe(function (nextResult) {
      setResult(nextResult);
    });
    (0, _queryCache.invalidateCachedObservableQuery)(client, query, restOptions);
    return function () {
      subscription.unsubscribe();
    };
  }, [query, (0, _objToKey.default)(restOptions)]);
  ensureSupportedFetchPolicy(restOptions.fetchPolicy, suspend);
  var helpers = {
    fetchMore: function fetchMore(opts) {
      return observableQuery.current.fetchMore(opts);
    },
    refetch: function refetch(variables) {
      return observableQuery.current.refetch(variables);
    },
    startPolling: function startPolling(interval) {
      return observableQuery.current.startPolling(interval);
    },
    stopPolling: function stopPolling() {
      return observableQuery.current.stopPolling();
    },
    updateQuery: function updateQuery(updaterFn) {
      return observableQuery.current.updateQuery(updaterFn);
    }
  };

  if (!(query === previousQuery.current && (0, _reactFastCompare.default)(restOptions, previousRestOptions.current))) {
    previousQuery.current = query;
    previousRestOptions.current = restOptions;
    var watchedQuery = (0, _queryCache.getCachedObservableQuery)(client, query, restOptions);
    observableQuery.current = watchedQuery;
    var currentResult = watchedQuery.currentResult();

    if (currentResult.partial && suspend) {
      // throw a promise - use the react suspense to wait until the data is
      // available
      throw watchedQuery.result();
    }

    setResult(currentResult);
    return _extends({}, helpers, currentResult);
  }

  return _extends({}, helpers, result);
}

function useMutation(mutation, baseOptions) {
  var client = useApolloClient();
  return function (localOptions) {
    return client.mutate(_extends({
      mutation: mutation
    }, baseOptions, localOptions));
  };
}

function ensureSupportedFetchPolicy(fetchPolicy, suspend) {
  if (!suspend) {
    return;
  }

  if (fetchPolicy && fetchPolicy !== 'cache-first') {
    throw new Error("Fetch policy " + fetchPolicy + " is not supported without 'suspend: false'");
  }
}