"use strict";

exports.__esModule = true;
exports.getCachedObservableQuery = getCachedObservableQuery;
exports.invalidateCachedObservableQuery = invalidateCachedObservableQuery;

var _printer = require("graphql/language/printer");

var _objToKey = _interopRequireDefault(require("./objToKey"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

var cachedQueriesByClient = new WeakMap();

function getCachedObservableQuery(client, query, options) {
  var queriesForClient = getCachedQueriesForClient(client);
  var cacheKey = getCacheKey(query, options);
  var observableQuery = queriesForClient.get(cacheKey);

  if (observableQuery == null) {
    observableQuery = client.watchQuery(_extends({
      query: query
    }, options));
    queriesForClient.set(cacheKey, observableQuery);
  }

  return observableQuery;
}

function invalidateCachedObservableQuery(client, query, options) {
  var queriesForClient = getCachedQueriesForClient(client);
  var cacheKey = getCacheKey(query, options);
  queriesForClient.delete(cacheKey);
}

function getCachedQueriesForClient(client) {
  var queriesForClient = cachedQueriesByClient.get(client);

  if (queriesForClient == null) {
    queriesForClient = new Map();
    cachedQueriesByClient.set(client, queriesForClient);
  }

  return queriesForClient;
}

function getCacheKey(query, options) {
  return (0, _printer.print)(query) + "@@" + (0, _objToKey.default)(options);
}